﻿using System;
using System.Web;
using System.Web.UI;
using System.Collections.Generic;
using System.Collections;
using System.Web.Globalization;
using System.Collections.Specialized;
using System.Reflection;
using System.Configuration;
using System.Text;
using System.Web.UI.HtmlControls;



[assembly: WebResource("MVCnExt.Web.UI.Ext.Scripts.adapter.ext.ext-base.js", "application/x-javascript")]
[assembly: WebResource("MVCnExt.Web.UI.Ext.Scripts.ext-all-debug.js", "application/x-javascript")]
[assembly: WebResource("MVCnExt.Web.UI.Ext.Scripts.ext-all.js", "application/x-javascript")]
[assembly: WebResource("MVCnExt.Web.UI.Ext.Scripts.resources.css.ext-all.css", "text/css", PerformSubstitution = true)]

//for simplicity, use ext-all.js instead of ext-core.js which requires us to manage the script files for every component

//[assembly: WebResource("MVCnExt.Web.UI.Ext.Scripts.ext-core-debug.js", "application/x-javascript")]
//[assembly: WebResource("MVCnExt.Web.UI.Ext.Scripts.ext-core.js", "application/x-javascript")]
namespace MVCnExt.Web.UI.Ext
{
    /// <summary>
    /// Acts as ScriptManager for pages without &lt;form runat="server"&gt; element or pages
    /// which will not be using native ASP.NET AJAX scripts.
    /// </summary>
    public class ExtScriptManager : ScriptManagerBase
    {
        private List<ScriptDescriptor> _descriptors = new List<ScriptDescriptor>();
        private List<ScriptDescriptor> _functions = new List<ScriptDescriptor>();
        private List<ExtScript> _scripts = new List<ExtScript>();

        public ExtScriptManager()
        {

        }

        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);

            if (!base.DesignMode)
            {
                Page.InitComplete += new EventHandler(OnPageInitComplete);
                Page.PreRenderComplete += new EventHandler(OnPagePreRenderComplete);
            }
        }

        void OnPageInitComplete(object sender, EventArgs e)
        {
            if (base.DesignMode)
                return;

            RegisterFrameworkStyles(base.ClientScript);
        }

        private void OnPagePreRenderComplete(object sender, EventArgs e)
        {
            //add framework base dependencies
            AddFrameworkScripts();

            // add descriptor scripts
            StringBuilder script = new StringBuilder();

            String url = ClientScript.GetWebResourceUrl(typeof(ExtComponent),
                "MVCnExt.Web.UI.Ext.Scripts.resources.images.default.s.gif");

            script.AppendFormat("Ext.BLANK_IMAGE_URL =\"{0}\";", url);

            script.Append("Ext.QuickTips.init();");

#if DEBUG
            //for readability
            script.AppendLine();
#endif
            foreach (ScriptDescriptor desc in _functions)
            {
                // append the client-side initialization script
                script.AppendLine(((ScriptControlDescriptorBase)desc).AppendScript());
            }

            // begin startup script registration
            script.Append("Ext.onReady(function(){");

#if DEBUG
            //for readability
            script.AppendLine();
#endif
            foreach (ScriptDescriptor desc in _descriptors)
            {
                // get the client-side initialization script
                script.AppendLine(((ScriptControlDescriptorBase)desc).WriteScript());
            }

            foreach (ExtScript extScript in _scripts)
            {
                if (extScript == null)
                    continue;
                // get the client-side initialization script
                script.AppendLine(extScript.text);
            }

            // close Ext.onReady()
            script.Append("});");

#if DEBUG
            //for readability
            script.AppendLine();
#endif

            base.ClientScript.RegisterStartupScript(this.GetType(),
                this.ClientID + "PageScript",
                script.ToString(), true);
        }

        /// <summary>
        /// Called by IScriptControl objects to register client-side object definitions
        /// with the startup script.
        /// </summary>
        /// <param name="scriptControl">The IScriptControl to be registered</param>
        public override void RegisterScriptDescriptors(IScriptControl scriptControl)
        {
            IEnumerable<ScriptDescriptor> descriptors;

            if (scriptControl is ExtOnReady)
            {
                _scripts.Add(((ExtOnReady)scriptControl).script);

                // get the list of property descriptors from the control
                descriptors = scriptControl.GetScriptDescriptors();

                // add property descriptors to global (page) list
                foreach (ScriptDescriptor d in descriptors)
                    _functions.Add(d);
                
                return;
            }

            if (scriptControl is ExtJSON)
            {                
                // get the list of property descriptors from the control
                descriptors = scriptControl.GetScriptDescriptors();

                // add property descriptors to global (page) list
                foreach (ScriptDescriptor d in descriptors)
                    _functions.Add(d);

                return;
            }


            // get the list of property descriptors from the control
            descriptors = scriptControl.GetScriptDescriptors();

            // add property descriptors to global (page) list
            foreach (ScriptDescriptor d in descriptors)
                _descriptors.Add(d);
        }



        /// <summary>
        /// Registers ExtJs framework scripts with the page
        /// </summary>
        private void AddFrameworkScripts()
        {
            // add script framework script dependencies (base scripts only, 
            // control depencies should be added by IScriptControl)
            if (Page != null)
            {
                // get the list of framework scripts
                IList<ScriptReference> references = GetFrameworkScripts(ClientScript);

                // register each script reference returned
                RegisterScriptReferences(references);
            }
        }

        /// <summary>
        /// Registers ExtJs css files with the page
        /// </summary>
        internal static void RegisterFrameworkStyles(ClientScriptManagerBase ClientScript)
        {
            //TODO: using ext-all for convenience, but should use specific scripts/css eventually
            String url = ClientScript.GetWebResourceUrl(typeof(ExtPanel),
                "MVCnExt.Web.UI.Ext.Scripts.resources.css.ext-all.css");

            ClientScript.RegisterClientCssInclude(typeof(ExtPanel), "_extallcss", url);
        }

        /// <summary>
        /// Gets ExtJs framework js files for registration with the page
        /// </summary>
        internal static List<ScriptReference> GetFrameworkScripts(ClientScriptManagerBase ClientScript)
        {
            List<ScriptReference> references = new List<ScriptReference>();

            references.Add(new ScriptReference(ClientScript.GetWebResourceUrl(typeof(ExtScriptManager),
                "MVCnExt.Web.UI.Ext.Scripts.adapter.ext.ext-base.js")));
#if DEBUG
            references.Add(new ScriptReference(ClientScript.GetWebResourceUrl(typeof(ExtScriptManager),
                "MVCnExt.Web.UI.Ext.Scripts.ext-all-debug.js")));
#endif
#if !DEBUG
                references.Add(new ScriptReference(ClientScript.GetWebResourceUrl(typeof(ExtScriptManager),
                    "MVCnExt.Web.UI.Ext.Scripts.ext-all.js")));
#endif

            return references;
        }
    }
}
